@@ -19,7 +19,7 @@ from veeam_em.const import (
     API_VERSION, DEFAULT_DEBUG, DEFAULT_TIMEOUT, HORT_PLATFORMS, PACKAGE_NAME,
     VERIFY_SSL, __version__ as product_version)
 from veeam_em.exceptions import VeeamEmError
-from veeam_em.helper import debug_requests_on
+from veeam_em.helper import debug_requests_off, debug_requests_on
 
 
 class VeeamEm:
@@ -95,6 +95,9 @@ class VeeamEm:
         """Set debug value and turn on debug requests."""
         if value:
             debug_requests_on()
+
+        if self._debug:
+            debug_requests_off()
         self._debug = value
 
     @property
@@ -1042,6 +1045,7 @@ class VeeamEm:
         ref_type: str,
         ref_h_root_id: str,
         object_ref: str,
+        validate_h_root: bool = False,
     ) -> str:
         """Get Hierarchy object reference type.
 
@@ -1067,6 +1071,7 @@ class VeeamEm:
          object resides.
         :param object_ref: ObjectRef is an ID of the virtual infrastructure
          object itself: mo-ref or ID, depending on the virtualization platform.
+        :param validate_h_root: whether to validate if provided root exists
         :return: str
         """
         # validate platform
@@ -1081,7 +1086,8 @@ class VeeamEm:
             raise VeeamEmError('type error. choose from: %s' % _pt_s)
         # validate ref_h_root_id if root does not exist,
         # 400 will be thrown
-        _ = self.get_hierarchy_root(ref_h_root_id)
+        if validate_h_root:
+            _ = self.get_hierarchy_root(ref_h_root_id)
         return 'urn:{platform}:{type}:{h_root_id}.{object_ref}'.format(
             platform=ref_platform,
             type=ref_type,
@@ -1122,3 +1128,63 @@ class VeeamEm:
         data = self.request('/lookup', params=params)
         # TODO: verify response representation: collection, dict?
         return data
+
+    def get_backup_task_sessions(
+        self, page_size: int = 2000, **kwargs
+    ) -> List[Dict]:
+        """Get backup task sessions using the query service.
+
+        {'CreationTimeUTC': '2020-12-10T15:06:21.363Z',
+         'Href': '/api/backupTaskSessions/...?format=Entity',
+         'JobSessionUid': 'urn:veeam:BackupJobSession:...',
+         'Links': [{'Href': 'h/api/backupServers/-...',
+                    'Name': 'host.domain.ca',
+                    'Rel': 'Up',
+                    'Type': 'BackupServerReference'},
+                   {'Href': '/api/backupSessions/...',
+                    'Name': 'Backup Job Folder vm@2020-12-10 15:05:12',
+                    'Rel': 'Up',
+                    'Type': 'BackupJobSessionReference'},
+                   {'Href': '/api/backupTaskSessions/...',
+                    'Name': 'vm-3@2020-12-10 15:06:21',
+                    'Rel': 'Alternate',
+                    'Type': 'BackupTaskSessionReference'}],
+         'Name': 'vn-3@2020-12-10 15:06:21',
+         'Reason': '',
+         'Result': 'Success',
+         'State': 'InProgress',
+         'TotalSize': 42949672960,
+         'Type': 'BackupTaskSession',
+         'UID': 'urn:veeam:BackupTaskSession:...',
+         'VmDisplayName': 'vm-3',
+         'VmUid': 'urn:VMware:Vm:....vm-13587'}
+        """
+        params = dict(
+            type='BackupTaskSession',
+            format='Entities',
+            pageSize=page_size,
+            sortDesc='CreationTime',
+            page=1,
+        )
+        params.update(kwargs)
+        page_resource = '/query'
+        result = []
+        rv = self.request(page_resource, params=params)
+        while True:
+            data = rv['Entities']['BackupTaskSessions']['BackupTaskSessions']
+            if not isinstance(data, list):
+                break
+            result.extend(data)
+            page_info = rv['PagingInfo']
+            if page_info:
+                pages_count = page_info.get('PagesCount')
+                if pages_count > 1:
+                    links = page_info.get('Links')
+                    next_url = [i for i in links if i['Rel'] == 'Next']
+                    if next_url:
+                        rv = self.request(next_url[0]['Href'])
+                    else:
+                        break
+                else:
+                    break
+        return result
